#!/usr/bin/python3
# coding: utf-8
#
#   createAndChangeCaseDialog.py
#
#       caseの作成、solver、meshの入れ替え
#
#   13/03/31    新規作成
#      04/06    空patchのboundaryFieldクリア追加
#      04/08    openFolderを追加
#      04/10    現在caseのtimeFolder内のメッシュ入れ替えを追加
#      04/15    newCaseの作成場所の指定を追加
#      04/20    ボタンの動作を変更
#      04/27    notebook方式に変更、設定を記憶させる
#               cell数が同じメッシュを入れ替えた場合、internalFieldをクリアしない。
#               timeFolder内のpolyMeshは、初めの10ヶと最後の10ヶをチェックする。
#      05/08    meshの一部入れ替えの場合エラー発生。これを修正
#      05/09    mesh入れ替え時の方向icon追加
#      06/09    tutorialsCaseのチェックをゆるめる。
#      07/14    meshDirの検索方法をglobに変更
#      07/20    mesh入れ替え:コピー先のdirも選択できるように修正
#      07/21    mesh入れ替え:dirを選択した場合、反対側のボタンを非enableに設定
#      07/27    mesh入れ替え:表示を変更
#      08/15    newCase,solver入れ替えの表示方法修正
#      08/21    clearOnlyNullBoundaryFields:fieldNamesの取得方法変更
#               バグ修正：隠しファイルが有った場合、エラー発生
#      09/6     newCase作成時、case名を入力できる様修正
#      09/25    meshCopy時:binaryMeshのコピーに対応
#      10/04    multiRegionの場合、meshを一括コピーできる様に対応
#      10/05    OpenFOAM用のcaseでない場合、pane_2, pane_3をunenableに設定
#      10/15    国際化による修正
#      12/21    stdout、stderrの設定（import logFileCreater）を追加
#   14/03/01    case作成時、currentCaseのtimeフォルダが存在しない場合エラーが発生
#               していたので、修正。addNullTimeFolder内を修正
#      09/02    clearNullBoundaryFieldsInternalFields:バグ修正
#      09/22    addNullTimeFolder:nullTimeはfirstTimeがmustに変更
#      10/23    copyMesh:cell数が異なる場合、internalFieldをクリアするか訊く様に修正
#      11/09    copyMesh:checkCurrMeshを追加
#      12/22    readMeshLocation,getMeshToCopyCase:「constant」をmeshが無くても追加
#   15/02/12    getMeshToCopyCase:バグ修正（ｺﾋﾟｰ先のdirを選択時ｴﾗｰ発生）
#      07/13    clipBoardの場所をTreeFoam/dataに変更。initialize部を修正。
#      07/17    dialog起動方法修正
#   17/02/26    initialize:solverの有無でfoamCaseかどうかを確認していたが、
#               constant, systemフォルダの有無で確認するように修正
#      04/15    getNCellPolyMesh:owner, neibourのcheckからowner側のみのcheckに変更
#      09/08    currCaseDirを引数で取得するように修正
#               referenceDir:相対参照でdirを取得する方法を変更。
#   19/05/22    selectCase,copyFieldParentToRegion:mkdirをos.mkdirに変更
#               cpをshutil.copyに変更
#      09/15    python3, GTK+3用に大幅修正
#   20/04/20    多言語化対応
#      09/12    main:entry_caseNameを非active設定をactive設定に戻す。
#      09/15    copyOtherCase:progressBar.pasteCaseLocalToLocalの引数にnewCaseName追加
#               終了時にnewCaseのpasteDirをtempFolderに保存
#

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk
import os, sys
import shutil
import glob
import pyTreeFoam
from createNewCaseDDialog import Show as createNewCaseDDialogShow
from getOpenFolderDDialog import Show as getOpenFolderDDialogShow
import progressBar
import universalDialogs


import logFileCreater
logFileCreater.log()

createNewCase_data = os.getenv("TreeFoamUserPath") + "/data/createNewCase_data"


#  ---------
#  windowApp
#  ---------
class windowApp:

    def __init__(self, caseDir):
        self.builder = Gtk.Builder()
        self.builder.set_translation_domain("treefoam")
        path = os.getenv("TreeFoamPath") + os.sep + "glade" + os.sep
        self.builder.add_from_file(path+"createAndChangeCaseDialog.glade")
        self.mainWindow = self.builder.get_object("window1")
        self.mainWindow.connect("delete-event", self.close)
        self.builder.connect_signals(self)
        #変数の設定
        self.caseDir = caseDir
        self.copyCase = ""
        self.pasteCase = ""
        self.copiedFlag = "no"          #copyButtonをclickしたかどうか
        #GUIのobject名を設定
        self.setGtkObject()
        #meshをlistBoxに設定
        self.iniSetMeshesTolistBox()
        #dialogの初期設定（createNewCase_data読み込み）
        self.iniDialog()
        #newCaseのpasteDir名の保存先(treefoam.py側で読み込む)
        self.savePasteDir = os.getenv("TreeFoamUserPath") + "/temp/newCasePasteDir"


    def setGtkObject(self):
        """ glade内のobject名を取得"""
        self.noteBook = self.builder.get_object("noteBook")
        #newCase作成
        self.radio_Tutorial = self.builder.get_object("radiobutton2")
        self.radio_otherCase = self.builder.get_object("radiobutton1")
        self.entry_tutoDir = self.builder.get_object("entry_tutoDir")
        self.entry_pasteDir = self.builder.get_object("entry_pasteDir")
        self.entry_caseName = self.builder.get_object("entry_caseName")
        #mesh入れ替え
        self.label_copyCase = self.builder.get_object("label_copyCase")
        self.label_pasteCase = self.builder.get_object("label_pasteCase")
        self.tree_copy = self.builder.get_object("tree_copy")
        self.tree_paste = self.builder.get_object("tree_paste")
        self.box_changeMesh = self.builder.get_object("box_changeMesh")
        self.button_changeCopyCase = self.builder.get_object("button_changeCopyCase")
        self.button_changePasteCase = self.builder.get_object("button_changePasteCase")
        #treeの設定
        treeselection = self.tree_copy.get_selection()
        treeselection.set_mode(Gtk.SelectionMode.MULTIPLE)

    def main(self):
        """ GUIの表示"""
        self.entry_pasteDir.set_text(self.caseDir)
        #self.entry_caseName.set_sensitive(False)
        self.mainWindow.show()
        Gtk.main()
        #pasteDirを保存
        if self.copiedFlag == "yes":
            pasteDir = self.entry_pasteDir.get_text()
        else:
            #copyButtonをクリックしていない場合
            pasteDir = ""
        f = open(self.savePasteDir, "w"); f.write(pasteDir); f.close()

    def close(self,*args):
        """ 閉じる"""
        Gtk.main_quit()

    def iniDialog(self):
        """ createNewCase_dataの内容をセット"""
        selNoteNo = 0
        newCase_tuto = "yes"
        if len(glob.glob(createNewCase_data)) != 0:
            f = open(createNewCase_data); lines = f.readlines(); f.close()
            for line in lines:
                words = line.split()
                if words[0] == "selNote":
                    try:
                        selNoteNo = int(words[1])
                    except:
                        selNoteNo = 0
                elif words[0] == "newCase_tutorials":
                    newCase_tuto = words[1]
        self.noteBook.set_current_page(selNoteNo)
        if newCase_tuto == "yes":
            self.radio_Tutorial.set_active(True)
        else:
            self.radio_otherCase.set_active(True)

        

    #  ----- event handler ---------------
    #----- 共通
    #close
    def onClose(self, event):
        self.ending()
        self.close()
    #----- caseCopy
    #getCopyCaseDir
    def onGetCopyCaseDir(self, event):
        self.getCopyDir()
    #getCreateCaseDir
    def onGetCreateCaseDir(self, event):
        self.getPasteDir()
    #startCopyCase
    def onStartCopyCase(self, event):
        self.startCopyCase()
    #----- mesh入れ替え
    def onChangeCopyCase(self, event):
        self.changeCopyCase()
    def onChangePasteCase(self, event):
        self.changePasteCase()
    def onStartMeshCopy(self, event):
        self.startMeshCopy()
    #  -----------------------------------

    #  -------------- mesh入れ替え ---------------
    #
    #  iniSetMeshesTolistBox
    #
    def iniSetMeshesTolistBox(self):
        if pyTreeFoam.getFolderType(self.caseDir) == "case":
            self.box_changeMesh.set_sensitive(True)
            #mesh読み込み
            meshDirs = getAllPolyMeshDirsInCase(self.caseDir)
            meshDirs.sort()
            #GUI側を修正
            caseName = self.caseDir.split("/")[-1]
            self.label_copyCase.set_text("caseName:"+caseName)
            self.label_pasteCase.set_text("caseName:"+caseName)
            #copyMeshesを作成
            self.setCopyMeshesToListBox(self.caseDir, meshDirs)
            #pasteMeshesを作成
            self.setPasteMeshesToListBox(self.caseDir, meshDirs)
            self.copyCase = self.caseDir
            self.pasteCase = self.caseDir
        else:
            #内容を隠す
            self.box_changeMesh.set_sensitive(False)

    def setCopyMeshesToListBox(self, caseDir, meshDirs):
        meshes = []
        for meshDir in meshDirs:
            polyMesh = meshDir[len(caseDir)+1:]
            meshes.append(polyMesh+"/polyMesh")
        self.tree_copy = self.setDataToListBox(self.tree_copy, meshes)

    def setPasteMeshesToListBox(self, caseDir, meshDirs):
        meshes = []
        for meshDir in meshDirs:
            polyMesh = meshDir[len(caseDir)+1:]
            meshes.append(polyMesh)
        #constantフォルダをチェックし、無ければ追加
        regions = []
        numFolders = []
        cFlag = 0
        for mesh in meshes:
            words = mesh.split("/")
            if words[0] == "constant":
                cFlag = 1
            if len(words) >= 2:
                #regionを取得
                regions.append(words[1])
            try:
                #timeFolder有無確認
                a = float(words[0])
                numFolders.append([a, words[0]])
            except:
                pass 
        if cFlag == 0:
            #constantフォルダを追加
            meshes.append("constant")
        #計算startFolderを取得
        currTimeFolder = pyTreeFoam.case(caseDir).getCurrTimeFolder()
        if len(numFolders) == 0:
            #timeFolder内にmeshが存在しない場合、
            #  meshの無いtimeFolderを追加
            meshes.append(currTimeFolder)
            if len(regions) > 0:
                #regionも追加
                regions = list(set(regions))
                regions.sort()
                for region in regions:
                    meshes.append(currTimeFolder+"/"+region)
        else:
            #timeFolder内にmeshが存在する場合
            #  currTimeFolder内にmeshがあるか確認
            flag = 0
            for mesh in meshes:
                words = mesh.split("/")
                if words[0] == currTimeFolder:
                    flag = 1
                    break
            if flag == 0:
                #currTimeFolder内にmeshがない場合
                meshes.append(currTimeFolder)
                if len(regions) > 0:
                    #regionがあれば追加
                    regions = list(set(regions))
                    regions.sort()
                    for region in regions:
                        meshes.append(currTimeFolder+"/"+region)
        self.tree_paste = self.setDataToListBox(self.tree_paste, meshes)

    def setDataToListBox(self, listBox, items):
        """ listBox(TreeView)にデータをセットし、項目を選択する。"""
        listmodel = Gtk.ListStore(str)
        for name in items:
            listmodel.append([name])
        listBox.set_model(model=listmodel)
        #columnを削除
        cols = listBox.get_columns()
        for _i in range(len(cols)):
            col = listBox.get_column(0)
            listBox.remove_column(col)
        #columnをappend
        cell = Gtk.CellRendererText()
        col = Gtk.TreeViewColumn("", cell, text=0)
        listBox.append_column(col)
        #treeViewのheaderを隠す
        listBox.set_headers_visible(False)
        if len(items) == 0:
            return listBox
        #第1番目を選択する
        path = Gtk.TreePath.new_from_indices([0])
        listBox.get_selection().select_path(path)
        return listBox
  
    #
    #  changeCopyCase
    #
    def changeCopyCase(self):
        """ copyCaseを変更する"""
        #変更先のcopyCaseDirを取得
        rootDir = pyTreeFoam.readConfigTreeFoam()["rootDir"]
        currDir = self.caseDir
        selDir = self.copyCase
        title = _("copy元のcase選択")
        mess = _("copy元（source側）のcaseを選択してください。")
        copyCase = getOpenFolderDDialogShow(title, mess, rootDir, currDir, selDir)
        if copyCase == "":
            return
        #pasteのbutton非表示設定
        if copyCase == self.caseDir:
            self.button_changePasteCase.set_sensitive(True)
        else:
            self.button_changePasteCase.set_sensitive(False)
        #meshDirsを取得
        meshDirs = getAllPolyMeshDirsInCase(copyCase)
        meshDirs.sort()
        #GUIを設定
        caseName = copyCase.split("/")[-1]
        self.label_copyCase.set_text("caseName:"+caseName)
        self.setCopyMeshesToListBox(copyCase, meshDirs)
        self.copyCase = copyCase

    #
    #  changePasteCase
    #
    def changePasteCase(self):
        """ pasteCaseを変更する"""
        #変更先のpasteCaseDirを取得
        rootDir = pyTreeFoam.readConfigTreeFoam()["rootDir"]
        currDir = self.caseDir
        selDir = self.pasteCase
        title = _("copy先のcase選択")
        mess = _("copy先（paste側）のcaseを選択してください。")
        pasteCase = getOpenFolderDDialogShow(title, mess, rootDir, currDir, selDir)
        if pasteCase == "":
            return
        #copyのbutton非表示設定
        if pasteCase == self.caseDir:
            self.button_changeCopyCase.set_sensitive(True)
        else:
            self.button_changeCopyCase.set_sensitive(False)
        #meshDirsを取得
        meshDirs = getAllPolyMeshDirsInCase(pasteCase)
        meshDirs.sort()
        #GUIを設定
        caseName = pasteCase.split("/")[-1]
        self.label_pasteCase.set_text("caseName:"+caseName)
        self.setPasteMeshesToListBox(pasteCase, meshDirs)
        self.pasteCase = pasteCase

    #
    #  startMeshCopy
    #
    def startMeshCopy(self):
        if self.copyCase == self.pasteCase:
            title = _("エラー")
            mess = _("copy元とpaste先が同じdirectoryです。") + "\n"
            mess += _("  コピーできません")
            print(mess)
            self.errDialog(title, mess)
            return
        #選択しているmeshを取得
        selCopyMeshes = self.getSelectMeshes(self.tree_copy)
        selPasteMeshes = self.getSelectMeshes(self.tree_paste)
        if len(selPasteMeshes) != 1:
            title = _("エラー")
            mess = _("コピー先は、複数選択できません。")
            self.errDialog(title, mess)
            return

        #コピー元を複数選択している場合
        if len(selCopyMeshes) > 1:
            pasteName = selPasteMeshes[0]
            if len(pasteName.split("/")) != 1:
                mess = _("コピー元を複数選択する場合は、コピー先を「constant」または「timefilder」に設定してください。")
                self.errDialog(_("エラー"), mess)
                return
            #エラーチェック
            errFlag = 0
            for selCopyMesh in selCopyMeshes:
                meshLoc = selCopyMesh.split("/")[-2]
                if meshLoc == "constant":
                    errFlag = 1
                else:
                    try:
                        float(meshLoc)
                        errFlag = 1
                    except:
                        pass
                if errFlag == 1:
                    break
            if errFlag == 1:
                mess = _("コピー元を複数選択する場合は、regionのメッシュを選択してください。")
                self.errDialog(_("エラー"), mess)
                return
            #copyMesh, pasteMeshのfullPathを取得
            copyMeshDirs = []
            pasteMeshDirs = []
            pasteDir = self.pasteCase + "/" + selPasteMeshes[0]
            for selCopyMesh in selCopyMeshes:
                meshDir = self.copyCase + "/" + selCopyMesh
                copyMeshDirs.append(meshDir)
                pasteMeshDir = pasteDir + "/" + selCopyMesh.split("/")[1]
                pasteMeshDirs.append(pasteMeshDir)
            #paste先のmeshを削除する
            for pasteMeshDir in pasteMeshDirs:
                if len(glob.glob(pasteMeshDir+"/polyMesh")) != 0:
                    shutil.rmtree(pasteMeshDir+"/polyMesh")
            #copy開始
            stat = progressBar.pasteLocalToLocal(copyMeshDirs, pasteMeshDirs).paste()
            if stat != "OK":
                return
            #regionPropertiesのコピー
            error = copyRegionProperties(self.copyCase, self.pasteCase)
            if error != "":
                mess = _("regionPropertiesがコピーできませんでした。\n新たにregionPopertiesを作成してください。")
                self.errDialog(_("エラー"), mess)
                return
            clearInternalBoundaryFieldMultiPolyMesh(self.pasteCase, pasteDir,
                    copyMeshDirs)
            mess = _("regionのメッシュをコピーしました。")
            self.okDialog(_("meshの入れ替え"), mess)
            return

        #コピー元が1ヶのpolyMeshの場合
        if len(selPasteMeshes) != 1:
            title = _("エラー")
            mess = _("コピー先を1ヶ選択してください")
            self.errDialog(title, mess)
            return
        #fullPathを取得
        copyMeshDir = self.copyCase + os.sep + selCopyMeshes[0]
        pasteMeshDir = self.pasteCase + os.sep + selPasteMeshes[0]
        if copyMeshDir == pasteMeshDir + "/polyMesh":
            mess = _("コピー元とコピー先が同じ設定になっています。")
            self.errDialog(_("エラー"), mess)
            return
        #cell数を取得
        print("getting number of cells in source mesh...")
        nCellCopy = self.getNCellPolyMesh(self.copyCase, copyMeshDir)
        print("getting number of cells in paste mesh...")
        nCellPaste = self.getNCellPolyMesh(self.pasteCase, pasteMeshDir)
        #paste先のpolyMeshを削除する。
        if len(glob.glob(pasteMeshDir+"/polyMesh")) != 0:
            shutil.rmtree(pasteMeshDir+"/polyMesh")
        #コピー開始
        stat = progressBar.pasteLocalToLocal([copyMeshDir], [pasteMeshDir]).paste()
        if stat != "OK":
            return
        #boundaryFieldとinternalFieldを整形
        if nCellCopy != nCellPaste and nCellCopy != 0:
            #cell数が違う場合（異なるmeshの場合）
            title = _(u"メッシュの入れ替え")
            mess = _(u"コピー元とコピー先のcell数が異なっています。\n")
            mess += _(u"コピー後のメッシュに合わせる為、")
            mess += _(u"internalFieldをクリアしますか？\n")
            mess += _(u"（nonuniform形式をuniform形式にクリアする。）")
            stat = self.yesNoDialog(title, mess)
            if stat == "YES":
                clearNullBoundaryFieldsInternalFields(self.pasteCase, pasteMeshDir)
            else:
                clearOnlyNullBoundaryFields(self.pasteCase, pasteMeshDir)
        else:
            #cell数が同じ場合（同じメッシュの場合）
            clearOnlyNullBoundaryFields(self.pasteCase, pasteMeshDir)

    def getNCellPolyMesh(self, caseDir, meshDir):
        """ cell数を取得。ouner, neighbourが存在しない場合は「0」を戻す"""
        #owner側のcellNoを確認
        fileName = meshDir + "/owner"
        fileName = pyTreeFoam.case(caseDir).checkFileName(fileName)
        if len(glob.glob(fileName)) == 0:
            return 0
        foam = pyTreeFoam.foamFile()
        (ascii, data) = foam.readSeparate(fileName)
        if foam.checkBinary(ascii) == "binary":
            numData = foam.structBinData(data[0])
            ownerMax = max(numData)
        else:
            sList = data[0][1].split()
            ownerMax = 0
            for n in sList:
                if len(n) > 0:
                    num = int(n)
                    if ownerMax < num:
                        ownerMax = num
            return ownerMax

    def getSelectMeshes(self, treeView):
        """ treeViewから選択している項目を返す"""
        treeStore = treeView.get_model()
        treeSelection = treeView.get_selection()
        _tree, pathes = treeSelection.get_selected_rows()
        selItems = []
        for path in pathes:
            treeiter = treeStore.get_iter(path)
            value = treeStore.get_value(treeiter, 0)
            selItems.append(value)
        return selItems


    #  ----------- newCase作成（tutorialsから）-------
    #
    #  getCopyDir
    #  ----------
    def getCopyDir(self):
        """ copyDirを取得して、textBoxに保存"""
        if self.radio_Tutorial.get_active() == True:
            #tutorialsから取得
            title = _("newCaseの作成")
            mess = _("tutorialsからcaseをコピーして、newCaseを作成します。\ncaseを選択してください。")
            currDir = self.caseDir
            [status, selCaseCopyDir] = createNewCaseDDialogShow(title, mess, currDir)
            if status != "OK":
                return ""
            self.entry_tutoDir.set_text(selCaseCopyDir)
            caseName = selCaseCopyDir.split("/")[-1]
            self.entry_caseName.set_text(caseName)
        else:
            #任意のcaseから取得
            currDir = self.caseDir
            title = _("newCaseの作成")
            mess = _("手本にするcaseを選択する")
            rootDir = pyTreeFoam.readConfigTreeFoam()["rootDir"]
            newCaseDir = getOpenFolderDDialogShow(title, mess, rootDir, currDir, currDir)
            if newCaseDir == "":
                return
            self.entry_tutoDir.set_text(newCaseDir)
            caseName = newCaseDir.split("/")[-1]
            self.entry_caseName.set_text(caseName)

    #
    #  getPasteDir
    #
    def getPasteDir(self):
        """ pasteDirを取得して、textBoxに保存"""
        title = _("newCaseの作成")
        cont = _("tutorialsからcaseをコピーして、newCaseを作成します。\ncaseを選択してください。")
        currDir = self.caseDir
        rootDir = pyTreeFoam.readConfigTreeFoam()["rootDir"]
        selDir = getOpenFolderDDialogShow(title, cont, rootDir, currDir, currDir)
        if selDir != "":
            self.entry_pasteDir.set_text(selDir)

    #
    #  startCopyCase
    #
    def startCopyCase(self):
        """ 選択したcaseを指定した場所にコピーする"""
        self.copiedFlag = "yes"
        copyCaseDir = self.entry_tutoDir.get_text()
        pasteDir = self.entry_pasteDir.get_text()
        newCaseName = self.entry_caseName.get_text()
        if pasteDir[0] != "/":
            pasteDir = caseDir + os.sep + pasteDir
        if copyCaseDir[0] != "/":
            copyCaseDir = caseDir + os.sep + copyCaseDir
        error = copyOtherCase(pasteDir, newCaseName, copyCaseDir)
        if error != "":
            title = _("エラー")
            self.errDialog(title, error)
        else:
            title = _("newCaseの作成")
            mess = _("caseをコピーして、newCaseを作成しました。")
            self.okDialog(title, mess)

    #  -------- ending -----------------
    #
    #  ending
    #
    def ending(self):
        selNoteNo = self.noteBook.get_current_page()
        if self.radio_Tutorial.get_active() == True:
            newCase_tuto = "yes"
        else:
            newCase_tuto = "no"
        cont  = "selNote " + str(selNoteNo) + "\n"
        cont += "newCase_tutorials " + newCase_tuto + "\n"
        f = open(createNewCase_data, "w"); f.write(cont); f.close()

    #  ------- universal dialog ---------------
    def okDialog(self, title, mess):
        universalDialogs.okDialog(self, title, mess)
    
    def errDialog(self, title, mess):
        universalDialogs.errDialog(self, title, mess)
    
    def yesNoDialog(self, title, mess):
        return universalDialogs.yesNoDialog(self, title, mess)


#
#  getAllPolyMeshDirsInCase
#
def getAllPolyMeshDirsInCase(caseDir):
    """ caseDir内の全polyMesh（region含む）を取得する"""
    meshDirs = getConstantPolyMesh(caseDir)
    meshDirs += getTimePolyMeshDirs(caseDir)
    return meshDirs

#  getConstantPolyMesh
#
def getConstantPolyMesh(caseDir):
    """ constant内のpolyMesh（region含む）を取得する"""
    dirs = glob.glob(caseDir + "/constant/polyMesh/boundary")
    dirs += glob.glob(caseDir + "/constant/*/polyMesh/boundary")
    meshDirs = []
    for name in dirs:
        #polyMesh/boundaryを削除
        nameList = name.split("/")[:-2]
        meshDirs.append("/".join(nameList))
    meshDirs.sort()
    return meshDirs

#  getTimePolyMeshDirs
#
def getTimePolyMeshDirs(caseDir):
    """ timeFolder内のpolyMesh（region含む）を取得する"""
    dirs = glob.glob(caseDir + "/*/polyMesh/boundary")
    dirs += glob.glob(caseDir + "/*/*/polyMesh/boundary")
    aMeshDirs = []
    for name in dirs:
        nameList = name[len(caseDir)+1:].split("/")[:-2]
        try:
            a = float(nameList[0])
            aMeshDirs.append([a, nameList])
        except:
            pass
    aMeshDirs.sort()
    meshDirs = []
    for aMeshDir in aMeshDirs:
        meshDir = caseDir + "/" + "/".join(aMeshDir[1])
        meshDirs.append(meshDir)
    return meshDirs
 

#
#  copyOtherCase
#
def copyOtherCase(pasteDir, newCaseName, copyCaseDir):
    """ createNewCaseの為のcaseCopy"""
    error = ""
    if len(glob.glob(pasteDir)) == 0:
        error = _("dirが選択されていません。")
        print(error)
        return error
    if len(glob.glob(copyCaseDir)) == 0:
        error =  _("選択した「") + copyCaseDir + _("」は、存在しません。")
        print(error)
        return error
    #コピー開始
    stat = progressBar.pasteCaseLocalToLocal([copyCaseDir], [pasteDir], newNames=[newCaseName]).paste()
    if stat == "OK":
        error = ""
    else:
        error = _("caseCopy時にエラーが発生しました")
    return error

#  copyRegionProperties
#    regionPropertiesのコピー
def copyRegionProperties(selCaseDir, toCopyCaseDir):
    """ regionPropertiesをコピーする"""
    copyFile = selCaseDir + "/constant/regionProperties"
    pasteDir = toCopyCaseDir + "/constant"
    error = ""
    if len(glob.glob(copyFile)) == 0:
        error = "NG"
    else:
        shutil.copy(copyFile, pasteDir)
        error = ""
    return error

#
#  clearInternalBoundaryFieldMultiPolyMesh
#    複数polyMeshをコピーしたときのinternal, boundaryFieldクリア
def clearInternalBoundaryFieldMultiPolyMesh(caseDir, meshDir, sources):
    """ 複数polyMeshをコピーした時、internal, boundaryFieldクリアする"""
    regionDirs = glob.glob(meshDir + "/*/polyMesh")
    regions = []
    for regionDir in regionDirs:
        region = regionDir.split("/")[-2]
        regions.append(region)
    currTime = pyTreeFoam.case(caseDir).getCurrTimeFolder()
    i = 0
    for region in regions:
        fieldDir = caseDir + "/" + currTime + "/" + region
        if len(glob.glob(fieldDir)) == 0:
            #timeFolderにregionがない場合、fieldをコピーする
            copyFieldParentToRegion(caseDir+"/"+currTime, region)
        fields = pyTreeFoam.case(caseDir).getFieldNames(currTime, region)
        polyMeshDir = meshDir + "/" + region
        setCase = pyTreeFoam.case(caseDir)
        setCase.clearNullBoundaryFields(currTime, polyMeshDir, fields, region)
        setCase.clearInternalFields(fieldDir, fields)
        i += 1
    return

#
#  copyFieldParentToRegion
#    timeFolderのfieldをregionにコピーする
def copyFieldParentToRegion(parentDir, region):
    files = glob.glob(parentDir + "/*")
    os.mkdir(parentDir + "/" + region)
    for file in files:
        if os.path.isfile(file) == True:
            shutil.copy(file, parentDir + "/" + region)

#
#  clearNullBoundaryFieldsInternalFields
#
def clearNullBoundaryFieldsInternalFields(caseDir, polyMeshDir):
    """ メッシュ入れ替え時のnullpatchのみboundaryFieldとinternalFieldをクリアする"""
    #fieldDir, fieldsを取得
    foamCase = pyTreeFoam.case(caseDir)
    time = foamCase.getCurrTimeFolder()
    region = polyMeshDir.split("/")[-1]
    if region == "constant":
        region = "."
        fieldDir = caseDir + "/" + time
    else:
        try:
            float(region)
            region = "."
        except:
            pass
        fieldDir = caseDir + "/" + time + "/" + region
    fields = foamCase.getFieldNames(time, region)
    #currMeshDirの取得
    relativeCurrPolyMeshDir = foamCase.getCurrMeshDir(time, region, "boundary")
    currPolyMeshDir = caseDir + "/" + relativeCurrPolyMeshDir
    meshDir = "/".join(currPolyMeshDir.split("/")[:-1])
    #boundaruFieldのクリア
    foamCase.clearNullBoundaryAllFields(time, meshDir, region)
    #internalFieldのクリア
    foamCase.clearInternalFields(fieldDir, fields)
    return

#
#  clearOnlyNullBoundaryFields
#
def clearOnlyNullBoundaryFields(caseDir, polyMeshDir):
    """ メッシュ入れ替え時のnullPatchのみboundaryfieldをクリアする"""
    #fieldDir, fieldsを取得
    foamCase = pyTreeFoam.case(caseDir)
    time = foamCase.getCurrTimeFolder()
    region = polyMeshDir.split("/")[-1]
    if region == "constant":
        region = "."
        fieldDir = caseDir + "/" + time
    else:
        try:
            float(region)
            region = "."
        except:
            pass
        fieldDir = caseDir + "/" + time + "/" + region
    fields = foamCase.getFieldNames(time, region)
    #currMeshDirの取得
    relativeCurrPolyMeshDir = foamCase.getCurrMeshDir(time, region, "boundary")
    currPolyMeshDir = caseDir + "/" + relativeCurrPolyMeshDir
    meshDir = "/".join(currPolyMeshDir.split("/")[:-1])
    foamCase.clearNullBoundaryFields(time, meshDir, fields, region)






if __name__ == "__main__":
    import gettext
    import locale
    localeDir = os.getenv("TreeFoamPath") + "/data/locale"
    locale.bindtextdomain("treefoam", localeDir)
    gettext.install("treefoam", localeDir)
    #_ = gettext.gettext

    caseDir = sys.argv[1]      #作成場所
    #os.environ["TreeFoamPath"] = "/home/caeuser/TreeFoam-glade3"
    #caseDir = "/home/caeuser/CAE/CAE-FOAM/OF-v1906/pitzDaily"
    winApp = windowApp(caseDir)
    winApp.main()
