#!/usr/bin/python3
# coding: utf-8
#
#   progressBar.py
#
#       進捗状況を表示するdialog
#
#   19/12/07    新規作成
#   20/04/22    多言語化対応
#      09/15    pasteCaseLocalToLocal.__init__:引数にnewNamesを追加。
#               pasteCaseLocalToLocal.getBasePathName:basePathNameを
#               newNames(self.newCaseNames)から取得。
#

import os
import subprocess
import shutil
import time

try:
    import pyTreeFoam
except:
    import python.pyTreeFoam as pyTreeFoam


#------------------------------
#  dialog class
#------------------------------
class dialog:
    """ 進捗状況表示dialog"""

    def showSettingDialog(self, title, setting):
        """ dialogを起動する
        """
        #dialogを起動
        path = os.getenv("TreeFoamPath") + "/python"
        comm = "python3 " + path + "/settingDialog2.py '" + title + "' " + "'" + setting + "'"
        self.settingDialog = subprocess.Popen(
            comm,
            shell=True,
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE)
        #dialogに初期値を出力
        self.numSettingDialog(0, "preparing...")

    def numSettingDialog(self, num, cont):
        """ 進捗状況を表示する。
        途中でCANCELまたは、dialogを閉じた場合は、BrokenPipeエラーが発生する。
        この場合、"CANCEL"が戻る。
        """
        stat = "OK"
        try:
            #subprosessのstdinへ(Value, lineCont)を出力
            self.settingDialog.stdin.write((str(num) + "," + cont + "\n").encode())
            self.settingDialog.stdin.flush()
        except:
            stat = "CANCEL"
        return stat

    def closeSettingDialog(self):
        """ dialogを閉じる。"""
        self.numSettingDialog(100, "close")
        self.settingDialog.stdin.write(("close\n").encode())
        if self.settingDialog.poll() == None:
            #BrokenPipeでは無い時にflushする
            self.settingDialog.stdin.flush()


#-------------------------
#  pasteLoacLToLocal class
#-------------------------
class pasteLocalToLocal:
    """ file, folderをpasteする。
    copyDisとpasteDirsからペアの(copy,paste)のdirを取得してpasteする。
    
    Attribute:
        copyDirs (list(str))    :copyするfile, folderのlist
        pasteDirs (list(str))   :pasteするdirのlist
        stat (str)              :"OK" or "CANCEL"
    """

    def __init__(self, copyDirs, pasteDirs):
        self.copyDirs = copyDirs
        self.pasteDirs = pasteDirs
        self.stat = "OK"

    def paste(self):
        """ 貼り付け"""

        def checkDirs():
            """ copyDirとpasteDirが同じ場合は、paste対象から除外する"""
            newCopyDirs = []; newPasteDirs = []
            for i in range(len(self.copyDirs)):
                copyDir = self.copyDirs[i]
                pasteDir = self.pasteDirs[i]
                if copyDir != pasteDir:
                    newCopyDirs.append(copyDir)
                    newPasteDirs.append(pasteDir)
            self.copyDirs = newCopyDirs
            self.pasteDirs = newPasteDirs

        def getBasePathName(copyDir, pasteDir):
            """ コピー先に同じfile名が存在する場合、file名に _copy0を追加する"""
            baseName = copyDir.split(os.sep)[-1]
            basePath = pasteDir + os.sep + baseName
            if os.path.exists(basePath) == True:
                #コピー先に同じfile名が存在する時
                i = 0
                basePath = pasteDir + os.sep + baseName + "_copy" + str(i)
                while os.path.exists(basePath) == True:
                    i += 1
                    basePath = pasteDir + os.sep + baseName + "_copy" + str(i)
            #新しいbaseNameを取得
            baseName = basePath.split(os.sep)[-1]
            return baseName
        
        def getNewPathName(copyDir, pasteDir, copyPathName, copyBaseName):
            """ file, folder名のpaste先pathを取得する
            
            Args:
                copyDir (str)       :self.copyDirsの要素
                pasteDir (str)      :paste先folderのpath
                copyPathName (str)  :copyするfile or folderのpath
                copyBaseName (str)  :copyDirのbaseName
            Returns:
                newCopyName (str)   :pasteするfileのpath"""
            baseName = copyPathName[len(copyDir):]
            newCopyName = pasteDir + os.sep + copyBaseName + baseName
            return newCopyName

        def getCopyFolderFileLinkNames(inNames, names):
            """ dirをコピーする場合。copyNameとpasteNameを取得する"""
            #変数を展開
            (copyDir, copyBaseName, pasteDir) = inNames
            (copyFiles, newFiles, copyFolders, newFolders,
                linkDir, newLinkDir) = names
            #paste側のdirを取得
            newDirName = pasteDir + os.sep + copyBaseName
            newFolders.append(newDirName)
            #copyするfolder内の内容を確認する
            for (root, dirs, files) in os.walk(copyDir):
                if self.stat == "CANCEL":
                    break
                #folderを確認
                for name in dirs:
                    if self.stat == "CANCEL":
                        break
                    dirName = root + os.sep + name
                    newDirName = getNewPathName(copyDir, pasteDir, dirName, copyBaseName)
                    #folderがlinkかどうか
                    if os.path.islink(dirName) == True:
                        #folderLinkの場合
                        linkDir.append(dirName)
                        newLinkDir.append(newDirName)
                    else:
                        #folderの場合
                        copyFolders.append(dirName)
                        newFolders.append(newDirName)
                    #GUIがcloseしたかどうか
                    self.stat = progress.numSettingDialog(0.0, _("コピーの準備中..."))
                #fileを確認
                for name in files:
                    if self.stat == "CANCEL":
                        break
                    copyFile = root + os.sep + name
                    newFile = getNewPathName(copyDir, pasteDir, copyFile, copyBaseName)
                    if os.path.islink(copyFile) == True:
                        linkDir.append(copyFile)
                        newLinkDir.append(newFile)
                    else:
                        copyFiles.append(copyFile)
                        newFiles.append(newFile)
                    #GUIの情報を設定-----------
                    Value = 0.0
                    labelCont = str(len(copyFiles))+" " + _("コピーの準備中...")
                    #GUIの状況を確認
                    self.stat = progress.numSettingDialog(Value, labelCont)
            #変数を格納
            names = (copyFiles, newFiles, copyFolders, newFolders,
                     linkDir, newLinkDir)
            return names

        def getNewFileName(copyFile, pasteDir):
            """ self.copyDirsの要素がfileの場合、
            そのfileと同名のfileがpasteDirに存在する時、file名を修正する"""
            baseName = copyFile.split(os.sep)[-1]
            newPathName = pasteDir + os.sep + baseName
            if os.path.exists(newPathName) == True:
                #同名のfileがpasteDir内に存在する場合
                words = copyFile.split(os.sep)
                baseName = words[-1]
                i = 0
                newPathName = pasteDir + os.sep + baseName + "_copy" + str(i)
                while os.path.exists(newPathName) == True:
                    i += 1
                    newPathName = pasteDir + os.sep + baseName + "_copy" + str(i)
            return newPathName


        #dirを確認する
        checkDirs()
        if len(self.copyDirs) == 0:
            return "OK"
        #progressDialogを表示-----------------
        progress = dialog()
        progress.showSettingDialog(_("folderの貼り付け"), "copying .... ")
        self.stat = progress.numSettingDialog(0, "setting...")
        #paste開始
        copyFiles = []
        newFiles = []
        copyFolders = []
        newFolders = []
        linkDir = []
        newLinkDir = []
        #copyDir毎に確認し、copyName、newPasteNameを取得する
        for i in range(len(self.copyDirs)):
            if self.stat == "CANCEL":
                break
            copyDir = self.copyDirs[i]
            pasteDir = self.pasteDirs[i]
            #paste先のbaseNameを確認（同名が存在する場合、修正）
            copyBaseName = getBasePathName(copyDir, pasteDir)
            if os.path.isdir(copyDir) == True:
                #dirの場合
                inNames = (copyDir, copyBaseName, pasteDir)
                names = (copyFiles, newFiles, copyFolders, newFolders,
                         linkDir, newLinkDir)
                #namesを取得
                names = getCopyFolderFileLinkNames(inNames, names)
            else:
                #ファイルのコピーの場合
                copyFiles.append(copyDir)
                newFile = getNewFileName(copyDir, pasteDir)
                newFiles.append(newFile)
        #増分を取得
        dValue = 1.0 / float(len(copyFiles) + len(linkDir) + 1) * 100.0
        #pasteDirにdirectoryを作成
        for folderDir in newFolders:
            if os.path.exists(folderDir) == False:
                os.makedirs(folderDir)
        #fileのコピー開始
        Value = 0.0
        errorFiles = []
        for i in range(len(copyFiles)):
            if self.stat == "CANCEL":
                break
            print("copy:", newFiles[i])
            try:
                #fileをcopyする
                shutil.copy(copyFiles[i], newFiles[i])
            except:
                errorFiles.append(copyFiles[i])
            #GUIの表示内容を更新-----------------
            Value += dValue
            cpName = copyFiles[i].split(os.sep)[-1]
            labelCont = _(" コピー中 ") + cpName + "..."
            self.stat = progress.numSettingDialog(Value, labelCont)
        #linkのコピー
        for i in range(len(linkDir)):
            if self.stat == "CANCEL":
                break
            linkto = os.readlink(linkDir[i])
            if linkto[0] != ".":
                #絶対pathのリンク
                try:
                    os.symlink(linkto, newLinkDir[i])
                    print(_("絶対参照で「") + newLinkDir[i] + _("」リンクを作成しました。"))
                except:
                    errorFiles.append(linkDir[i])
            else:
                #相対pathのリンク
                try:
                    os.symlink(linkto, newLinkDir[i])
                    print(_("相対pathで「") + linkto + _("」リンクを作成しました。"))
                except:
                    errorFiles.append(linkDir[i])
            #GUIの表示内容を更新------------
            Value += dValue
            cpName = newLinkDir[i].split(os.sep)[-1]
            labelCont = _(" コピー中 ") + cpName + "..."
            self.stat = progress.numSettingDialog(Value, labelCont)
        #エラーチェック
        if len(errorFiles) != 0:
            print(_("以下のファイルがコピーできませんでした。"))
            for errorFile in errorFiles:
                print("error:", errorFile)
        #GUIを閉じる------------------
        progress.closeSettingDialog()
        return self.stat


#-----------------------------
#  pasteCaseLocalToLocal class
#-----------------------------
class pasteCaseLocalToLocal:
    """ caseをコピーする。
    caseDirsとpasteDirsからペアの(copy,paste)のdirを取得してコピーする。
    
    Attribute:
        caseDirs (list(str))    :copyするcaseのlist
        pasteDirs (list(str))   :pasteするdirのlist
        stat (str)              :戻り値（"OK" or "CANCEL"）
    """
    def __init__(self, caseDirs, pasteDirs, newNames=[]):
        self.caseDirs = caseDirs
        self.pasteDirs = pasteDirs
        if len(newNames) == 0:
            #caseDirsのcase名を取得
            self.newCaseNames = list(map(lambda x: x.split("/")[-1], caseDirs))
        else:
            self.newCaseNames = newNames
        self.stat = "OK"

    def paste(self):
        """ local内のcaseをlocalのpasteDirへコピーする。
        self.caseDirsをself.pasteDirにコピーする。
        コピー先に同じcase名が存在する場合は、<case>_copy0 でコピーする。
        
        Returns:
            stat (str) :"OK" or "CANCEL(中断)"
        """
        
        def checkCaseDirs():
            """ caseDirとpasteDirが同じ場合は、copy対象から削除する"""
            newCaseDirs = []; newPasteDirs = []
            for i in range(len(self.caseDirs)):
                caseDir = self.caseDirs[i]
                pasteDir = self.pasteDirs[i]
                if caseDir != pasteDir:
                    newCaseDirs.append(caseDir)
                    newPasteDirs.append(pasteDir)
            self.caseDirs = newCaseDirs
            self.pasteDirs = newPasteDirs

        def getBasePathName(copyDir, pasteDir, newCaseName):
            """ コピー先に同じfile名が存在する場合、file名に _copy0を追加する"""
            #baseName = copyDir.split(os.sep)[-1]
            baseName = newCaseName
            basePath = pasteDir + os.sep + baseName
            if os.path.exists(basePath) == True:
                #コピー先に同じfile名が存在する時
                i = 0
                basePath = pasteDir + os.sep + baseName + "_copy" + str(i)
                while os.path.exists(basePath) == True:
                    i += 1
                    basePath = pasteDir + os.sep + baseName + "_copy" + str(i)
            #新しいbaseNameを取得
            baseName = basePath.split(os.sep)[-1]
            return baseName

        def getNewPathName(copyDir, pasteDir, copyPathName, copyBaseName):
            """ file, folder名のpaste先pathを取得する
            
            Args:
                copyDir (str)       :self.copyDirsの要素
                pasteDir (str)      :paste先folderのpath
                copyPathName (str)  :copyするfile or folderのpath
                copyBaseName (str)  :copyDirのbaseName
            Returns:
                newCopyName (str)   :pasteするfileのpath"""
            baseName = copyPathName[len(copyDir):]
            newCopyName = pasteDir + os.sep + copyBaseName + baseName
            return newCopyName

        def getCopyCaseFolderFileLinkNames(inNames, names):
            """ copyCase時のcopyNameとpasteNameを取得する"""
            #変数を展開
            (caseDir, caseBaseName, pasteDir) = inNames
            (copyFiles, newFiles, copyFolders, newFolders,
                linkDir, newLinkDir) = names
            #paste側のdirを取得する
            newCaseName = pasteDir + os.sep + caseBaseName
            newFolders.append(newCaseName)
            #copyするcase内の内容を確認する
            needFolders = pyTreeFoam.case(caseDir).getIniCaseFolders()
            for needFolder in needFolders:
                if self.stat == "CANCEL":
                    break
                #needFolderを確認
                dirName = needFolder
                newDirName = getNewPathName(caseDir, pasteDir, dirName, caseBaseName)
                #folderがlinkかどうか
                if os.path.islink(dirName) == True:
                    linkDir.append(dirName)
                    newLinkDir.append(newDirName)
                else:
                    copyFolders.append(dirName)
                    newFolders.append(newDirName)
                #needFolder内を確認
                for (root, dirs, files) in os.walk(needFolder):
                    if self.stat == "CANCEL":
                        break
                    #folderを確認
                    for name in dirs:
                        if self.stat == "CANCEL":
                            break
                        dirName = root + os.sep + name
                        newDirName = getNewPathName(caseDir, pasteDir, dirName, caseBaseName)
                        #folderがlinkかどうか
                        if os.path.islink(dirName) == True:
                            linkDir.append(dirName)
                            newLinkDir.append(newDirName)
                        else:
                            copyFolders.append(dirName)
                            newFolders.append(newDirName)
                        #GUIがcloseしたかどうか------
                        self.stat = progress.numSettingDialog(0.0, _("コピーの準備中..."))
                    #fileを確認
                    for name in files:
                        if self.stat == "CANCEL":
                            break
                        copyFile = root + os.sep + name
                        newFile = getNewPathName(caseDir, pasteDir, copyFile, caseBaseName)
                        if os.path.islink(copyFile) == True:
                            linkDir.append(copyFile)
                            newLinkDir.append(newFile)
                        else:
                            copyFiles.append(copyFile)
                            newFiles.append(newFile)
                        #GUIの情報を設定------------
                        Value = 0.0
                        labelCont = str(len(copyFiles))+" "+_("コピーの準備中...")
                        #GUIの状況を確認
                        self.stat = progress.numSettingDialog(Value, labelCont)
            #case直下のfileを確認
            needFiles = pyTreeFoam.case(caseDir).getIniCaseFiles()
            for name in needFiles:
                if self.stat == "CANCEL":
                    break
                copyFile = name
                newFile = getNewPathName(caseDir, pasteDir, copyFile, caseBaseName)
                if os.path.islink(copyFile) == True:
                    linkDir.append(copyFile)
                    newLinkDir.append(newFile)
                else:
                    copyFiles.append(copyFile)
                    newFiles.append(newFile)
                #GUIの情報を設定------------
                labelCont = str(len(copyFiles))+" "+_("コピーの準備中...")
                #GUIの状況を確認
                self.stat = progress.numSettingDialog(0.0, labelCont)
            #変数を格納
            names = (copyFiles, newFiles, copyFolders, newFolders,
                     linkDir, newLinkDir)
            return names

        #caseDirsを確認
        checkCaseDirs()
        if len(self.caseDirs) == 0:
            return "OK"
        #progressDialogを表示---------------
        #progress = progressBar.dialog()
        progress = dialog()
        progress.showSettingDialog(_("caseの貼り付け"), "copying .... ")
        self.stat = progress.numSettingDialog(0, "setting...")
        #paste開始
        copyFiles = []
        newFiles = []
        copyFolders = []
        newFolders = []
        linkDir = []
        newLinkDir = []
        #caseDir毎に確認し、copyName, newPasteNameを取得する
        for i in range(len(self.caseDirs)):
            if self.stat == "CANCEL":
                break
            caseDir = self.caseDirs[i]
            pasteDir = self.pasteDirs[i]
            newCaseName = self.newCaseNames[i]
            #paste先のbaseNameを確認（同名が存在する場合は修正）
            caseBaseName = getBasePathName(caseDir, pasteDir, newCaseName)
            if os.path.isdir(caseDir) == True:
                #caseDirの場合
                inNames = (caseDir, caseBaseName, pasteDir)
                names = (copyFiles, newFiles, copyFolders, newFolders,
                         linkDir, newLinkDir)
                #copyするfileNamesを取得する
                names = getCopyCaseFolderFileLinkNames(inNames, names)
        if self.stat == "CANCEL":
            return self.stat
        #増分を取得
        dValue = 1.0 / float(len(copyFiles) + len(linkDir) + 1) * 100.0
        #pasteDirにdirectoryを作成
        for folderDir in newFolders:
            if os.path.exists(folderDir) == False:
                os.makedirs(folderDir)
        #fileのコピー開始
        Value = 0.0
        errFiles = []
        for i in range(len(copyFiles)):
            if self.stat == "CANCEL":
                break
            print("copy:", newFiles[i])
            try:
                #fileをcopyする
                shutil.copy(copyFiles[i], newFiles[i])
            except:
                errFiles.append(copyFiles[i])
            #GUIの表示内容を更新------------------
            Value += dValue
            cpName = copyFiles[i].split(os.sep)[-1]
            labelCont = _(" コピー中 ") + cpName
            #GUIの状況を確認
            self.stat = progress.numSettingDialog(Value, labelCont)
        #linkのコピー
        for i in range(len(linkDir)):
            if self.stat == "CANCEL":
                break
            linkto = os.readlink(linkDir[i])
            if linkto[0] != ".":
                #絶対pathのリンク
                try:
                    os.symlink(linkto, newLinkDir[i])
                    print(_("絶対参照で「") + newLinkDir[i] + _("」リンクを作成しました。"))
                except:
                    errFiles.append(linkDir[i])
            else:
                #相対参照のリンク
                try:
                    os.symlink(linkto, newLinkDir[i])
                    print(_("相対pathで「") + linkto + _("」リンクを作成しました。"))
                except:
                    errFiles.append(linkDir[i])
            #GUIの表示内容を更新---------------
            Value += dValue
            cpName = newLinkDir[i].split(os.sep)[-1]
            labelCont = _(" コピー中 ") + cpName
            #GUIの状況を確認
            self.stat = progress.numSettingDialog(Value, labelCont)
        if self.stat == "CANCEL":
            return self.stat
        #エラーチェック
        if len(errFiles) != 0:
            print(_("以下のファイルがコピーできませんでした。"))
            for errFile in errFiles:
                print("error:", errFile)
        #GUIを閉じる---------
        #Value = 1.1
        progress.closeSettingDialog()
        return self.stat


#-------------------------
#  pasteServerLocal class
#-------------------------
class pasteServerLocal:
    """ serverを使ったコピー。
    localToServer, serverToLocal, serverToServerのコピー。
    local-server間はscpでコピー。server内同士はcpでコピーする。
    
    Attrubute:
        runFile (str)   :実行file名
        logFile (str)   :実行結果が記録されるfile
        stat (str)      :戻り値（"OK" or "CANCEL"）
        """

    def __init__(self, runFile, logFile):
        self.runFile = runFile
        self.logFile = logFile
        self.stat = "OK"

    def paste(self):
        """ serverを使ったコピー、caseコピーの進捗表示。
        localToServer, serverToLocal, serverToServerのコピー進捗表示。
        local-server間はscpでコピー、server内同士はcpでコピーしている。
        コピーの内容は、logFile内に記録されているので、この内容を
        確認しながら、進捗を表示させる。
        
        Returns:
            stat (str)      :progressDialogの状態("OK" or "CANCEL")
        """
        #dialogの設定-------------
        #self.progress = progressBar.dialog()
        self.progress = dialog()
        self.progress.showSettingDialog(_("server-local間コピー"), "copying .... ")
        self.stat = self.progress.numSettingDialog(0, _("serverを確認中..."))
        #logfileをopen
        self.logFileObj = open(self.logFile)
        #file数をlogFile中から読み取る
        nFiles = 0
        loop = True
        while loop:
            if self.stat == "CANCEL":
                break
            line = self.get1lineFromLogFile()
            if line != "":
                #GUI側へ表示--------
                labelCont = _("serverから取得中 " + str(nFiles))
                self.stat = self.progress.numSettingDialog(0.0, labelCont)
                if (line[:len("Script started")] == "Script started"):
                    loop = False
                    break
                nFiles += 1
            else:
                loop = False
                break
        #コピーのtype（scp、cp）を確認
        copyType = self.getCopyType_scp_or_cp()
        if copyType == "scp":
            #scp
            self.stat = self.copyFoldersFilesScp(nFiles)
        else:
            #cp
            self.stat = self.copyFoldersFilesCp(nFiles)
        #logFileをclose
        self.logFileObj.close()
        #dialogを閉じる
        self.progress.closeSettingDialog()
        return self.stat

    def get1lineFromLogFile(self):
        """ logFileから1行を取得する。
        logFileの更新が30s待っても更新されない場合は、「""」を返す。"""
        waitTime = 30.0
        count = 0.0
        loop = True
        while loop == True:
            line = self.logFileObj.readline()
            if line == "":
                time.sleep(0.2)
                count += 0.2
                if count >= waitTime:
                    loop = False
                    break
            else:
                loop = False
                break
        return line

    def getCopyType_scp_or_cp(self):
        """ copy方法が「scp」or 「cp」を調べる"""
        currPos = self.logFileObj.tell()
        loop = True
        copyType = ""
        while loop:
            line = self.get1lineFromLogFile()
            if line != "":
                words = line.split()
                if len(words) >= 3:
                    if words[1] == "100%":
                        copyType = "scp"
                        break
                    elif words[1] == "->":
                        copyType = "cp"
                        break
                    elif line[:len("Script done")] == "Script done":
                        copyType = ""
                        break
            else:
                break
        self.logFileObj.seek(currPos)
        return copyType

    #
    #  copyFoldersFilesScp
    #
    def copyFoldersFilesScp(self, nFiles):
        """ scpコピーの場合、進捗状況を取得"""
        nCopyFiles = 0
        loop = True
        while loop == True:
            if self.stat == "CANCEL":
                break
            line = self.get1lineFromLogFile()
            if line != "" and line[:len("Script done")] != "Script done":
                words = line.split()
                if len(words) > 3:
                    if words[1] == "100%":
                        cpName = words[0]
                        self.speed = words[3]
                        #率を計算（値が100以上にならない様に処理）
                        nCopyFiles += 1
                        value = nCopyFiles / nFiles * 100.0
                        if value >= 100.0:
                            value = 99.0
                        Value = value
                        labelCont = _(" コピー中 ") + cpName + " (" + self.speed + ")"
                        self.stat = self.progress.numSettingDialog(Value, labelCont)
            else:
                loop = False
                break
        return self.stat

    #
    #  copyFoldersFilesCp
    #
    def copyFoldersFilesCp(self, nFiles):
        """ cpコピーの場合、進捗状況を取得する"""
        nCopyFiles = 0
        loop = True
        while loop == True:
            if self.stat == "CANCEL":
                break
            line = self.get1lineFromLogFile()
            if line != "" and line[:len("Script done")] != "Script done":
                words = line.split()
                if len(words) > 1:
                    cpName = words[0][1:-1].split("/")[-1]
                    #率を計算（値が100以上にならない様に処理）
                    nCopyFiles += 1
                    value = nCopyFiles / nFiles * 100.0
                    if value >= 100.0:
                        value = 99.0
                    Value = value
                    labelCont = _(" コピー中 ") + cpName
                    self.stat = self.progress.numSettingDialog(Value, labelCont)
            else:
                loop = False
                break
        return self.stat


if __name__ == "__main__":
    import gettext
    localeDir = os.getenv("TreeFoamPath") + "/data/locale"
    gettext.install("treefoam", localeDir)
    #_ = gettext.gettext
